@@ -307,6 +307,14 @@ def get_command_line_arguments() -> argparse.Namespace:
         default=None,
         help="continue to read file if fatal errors",
     )
+    parser.add_argument(
+        "-g",
+        "--graph",
+        dest="graph",
+        action="store_true",
+        default=None,
+        help="generate graphs for documentation output",
+    )
     parser.add_argument(
         "--no-search",
         dest="search",
@@ -443,7 +451,6 @@ def parse_arguments(
         "css",
         "mathjax_config",
         "src_dir",
-        "exclude",
         "exclude_dir",
         "include",
     ]:
