@@ -25,9 +25,9 @@
 import os
 import pathlib
 import sys
-from multiprocessing import Pool
 
 from tqdm import tqdm
+from tqdm.contrib.concurrent import process_map
 
 from ford.sourceform import (
     FortranFunction,
@@ -99,9 +99,7 @@ class GraphManager(object):
             self.graph_objs.append(obj)
 
     def graph_all(self):
-        for obj in tqdm(
-            iter(self.graph_objs), file=sys.stdout, total=len(self.graph_objs), unit=""
-        ):
+        for obj in tqdm(self.graph_objs, unit="", desc="Generating graphs"):
             if isinstance(obj, FortranModule):
                 obj.usesgraph = ford.graphs.UsesGraph(obj, self.webdir)
                 obj.usedbygraph = ford.graphs.UsedByGraph(obj, self.webdir)
@@ -221,11 +219,12 @@ class GraphManager(object):
             )
             args.extend([(m.usesgraph, self.graphdir) for m in self.blockdata])
 
-            np = min(njobs, len(args))
-            pool = Pool(processes=np)
-            results = pool.map(outputFuncWrap, args, len(args) / np)  # noqa F841
-            pool.close()
-            pool.join()
+            process_map(
+                outputFuncWrap,
+                args,
+                max_workers=njobs,
+                desc="Writing graphs",
+            )
 
         if self.usegraph:
             self.usegraph.create_svg(self.graphdir)
