@@ -3,6 +3,7 @@ import ford
 import pathlib
 import pytest
 from textwrap import dedent
+import subprocess
 
 # Ford default src folder
 DEFAULT_SRC = "src"
@@ -19,7 +20,9 @@ def copy_file(data: str, path: pathlib.Path, filename: str) -> pathlib.Path:
 
 @pytest.fixture
 def copy_fortran_file(tmp_path):
-    return lambda data: copy_file(data, tmp_path / "src", "test.f90")
+    return lambda data, extension=".f90": copy_file(
+        data, tmp_path / "src", f"test{extension}"
+    )
 
 
 @pytest.fixture
@@ -38,3 +41,9 @@ def restore_macros():
 def restore_nameselector():
     yield
     ford.sourceform.namelist = ford.sourceform.NameSelector()
+
+
+def gfortran_is_not_installed():
+    """Returns False if gfortran is not (detectably) installed"""
+    out = subprocess.run("command -v gfortran", shell=True, check=False)
+    return out.returncode != 0
