@@ -7,6 +7,7 @@ from copy import deepcopy
 
 import markdown
 import pytest
+from bs4 import BeautifulSoup
 
 
 @pytest.fixture
@@ -538,6 +539,90 @@ def test_display_private_derived_types(copy_fortran_file):
     assert type_names == {"no_attrs", "public_attr", "private_attr"}
 
 
+def test_interface_type_name(copy_fortran_file):
+    """Check for shared prototype list"""
+    data = """\
+    module foo
+      type name_t
+      end type name_t
+
+      type(name_t) :: var, var2
+    end module foo
+    """
+
+    settings = copy_fortran_file(data)
+    project = create_project(settings)
+    proto_names = [var.proto[0].name for var in project.modules[0].variables]
+    assert proto_names == ["name_t", "name_t"]
+
+
+def test_imported_interface_type_name(copy_fortran_file):
+    """Check for shared prototype list"""
+    data = """\
+    module foo
+      type name_t
+      end type name_t
+    end module foo
+
+    module bar
+      use foo
+      type(name_t), allocatable :: var(:), var2(:)
+    end module bar
+    """
+
+    settings = copy_fortran_file(data)
+    project = create_project(settings)
+    proto_names = [var.proto[0].name for var in project.modules[1].variables]
+    assert proto_names == ["name_t", "name_t"]
+
+
+def test_abstract_interface_type_name(copy_fortran_file):
+    """Check for shared prototype list"""
+    data = """\
+    module foo
+      abstract interface
+        subroutine hello
+        end subroutine hello
+      end interface
+
+      procedure(hello) :: proc1, proc2
+    end module foo
+    """
+
+    settings = copy_fortran_file(data)
+    project = create_project(settings)
+    proto_names = [var.proto[0].name for var in project.modules[0].variables]
+    assert proto_names == ["hello", "hello"]
+
+
+def test_imported_abstract_interface_type_name(copy_fortran_file):
+    """Check for shared prototype list"""
+    data = """\
+    module foo
+      abstract interface
+        subroutine hello
+        end subroutine hello
+      end interface
+
+    contains
+      integer function goodbye()
+        goodbye = 1
+      end function goodbye
+    end module foo
+
+    module bar
+      use foo
+      procedure(hello) :: proc1, proc2
+      procedure(goodbye) :: proc3, proc4
+    end module bar
+    """
+
+    settings = copy_fortran_file(data)
+    project = create_project(settings)
+    proto_names = [var.proto[0].name for var in project.modules[1].variables]
+    assert proto_names == ["hello", "hello", "goodbye", "goodbye"]
+
+
 def test_display_internal_procedures(copy_fortran_file):
     """_very_ basic test of 'proc_internals' setting"""
 
@@ -601,8 +686,242 @@ def test_exclude(tmp_path):
 
     settings = deepcopy(DEFAULT_SETTINGS)
     settings["src_dir"] = [tmp_path]
-    settings["exclude"] = [normalise_path(tmp_path, "sub1/sub2/exclude.f90")]
+    settings["exclude"] = ["exclude.f90"]
     project = Project(settings)
 
     program_names = {program.name for program in project.programs}
     assert program_names == {"foo"}
+
+
+@pytest.mark.parametrize(
+    "sort_kind, expected_order",
+    [
+        (
+            "src",
+            {
+                "variables": ["b", "a", "d", "f", "c", "e", "g", "h", "j", "i"],
+                "types": ["bet", "alef", "gimel"],
+                "subroutines": ["bravo", "alpha", "charlie"],
+                "functions": ["beaver", "aardvark", "cat"],
+                "absinterfaces": ["apple", "carrot", "banana"],
+            },
+        ),
+        (
+            "alpha",
+            {
+                "variables": ["a", "b", "c", "d", "e", "f", "g", "h", "i", "j"],
+                "types": ["alef", "bet", "gimel"],
+                "subroutines": ["alpha", "bravo", "charlie"],
+                "functions": ["aardvark", "beaver", "cat"],
+                "absinterfaces": ["apple", "banana", "carrot"],
+            },
+        ),
+        (
+            "permission",
+            {
+                "variables": ["d", "f", "c", "e", "b", "a", "g", "h", "j", "i"],
+                "types": ["gimel", "alef", "bet"],
+                "subroutines": ["charlie", "alpha", "bravo"],
+                "functions": ["cat", "aardvark", "beaver"],
+                "absinterfaces": ["apple", "carrot", "banana"],
+            },
+        ),
+        (
+            "permission-alpha",
+            {
+                "variables": ["d", "f", "c", "e", "a", "b", "g", "h", "i", "j"],
+                "types": ["gimel", "alef", "bet"],
+                "subroutines": ["charlie", "alpha", "bravo"],
+                "functions": ["cat", "aardvark", "beaver"],
+                "absinterfaces": ["apple", "banana", "carrot"],
+            },
+        ),
+        (
+            "type",
+            {
+                "variables": ["h", "g", "j", "i", "b", "a", "c", "e", "d", "f"],
+                "types": ["bet", "alef", "gimel"],
+                "subroutines": ["bravo", "alpha", "charlie"],
+                "functions": ["cat", "beaver", "aardvark"],
+                "absinterfaces": ["apple", "carrot", "banana"],
+            },
+        ),
+        (
+            "type-alpha",
+            {
+                "variables": ["h", "g", "i", "j", "a", "b", "c", "e", "d", "f"],
+                "types": ["alef", "bet", "gimel"],
+                "subroutines": ["alpha", "bravo", "charlie"],
+                "functions": ["cat", "aardvark", "beaver"],
+                "absinterfaces": ["apple", "banana", "carrot"],
+            },
+        ),
+    ],
+)
+def test_sort(copy_fortran_file, sort_kind, expected_order):
+    data = """\
+    module mod_a
+      type :: bet
+      end type bet
+
+      type :: alef
+      end type alef
+
+      type :: gimel
+      end type gimel
+
+      integer, private :: b, a
+      type(gimel), public :: d, f
+      type(alef), protected :: c, e
+
+      character(len=2, kind=4) :: g
+      character(len=1, kind=4) :: h
+      character(len=1, kind=8) :: j, i
+      private :: g, h, i, j
+
+      abstract interface
+        subroutine banana
+        end subroutine banana
+
+        function apple()
+          import gimel
+          class(gimel), allocatable :: apple
+        end function apple
+
+        type(alef) function carrot()
+          import alef
+        end function carrot
+      end interface
+
+      public :: gimel, charlie, cat
+      protected :: alef, alpha, aardvark
+      private :: bet, bravo, beaver
+
+    contains
+      subroutine bravo
+      end subroutine bravo
+
+      subroutine alpha
+      end subroutine alpha
+
+      subroutine charlie
+      end subroutine charlie
+
+      type(gimel) function beaver()
+      end function beaver
+
+      function aardvark()
+        class(gimel), allocatable :: aardvark
+      end function aardvark
+
+      type(alef) function cat()
+      end function cat
+    end module mod_a
+    """
+
+    settings = copy_fortran_file(data)
+    settings["sort"] = sort_kind
+    settings["display"] = ["public", "private", "protected"]
+    project = create_project(settings)
+    module = project.modules[0]
+
+    for entity_type in [
+        "variables",
+        "types",
+        "subroutines",
+        "functions",
+        "absinterfaces",
+    ]:
+        entity_names = [entity.name for entity in getattr(module, entity_type)]
+        assert entity_names == expected_order[entity_type], (sort_kind, entity_type)
+
+
+def test_uses(copy_fortran_file):
+    """Check that module `USE`s are matched up correctly"""
+
+    data = """\
+    module mod_a
+    end module mod_a
+
+    module mod_b
+      use mod_a
+    end module mod_b
+
+    module mod_c
+      use mod_a
+      use mod_b
+    end module mod_c
+
+    module mod_d
+      use unquoted_external_module
+      use quoted_external_module
+    end module mod_d
+    """
+
+    extra_mods = {
+        "unquoted_external_module": "http://unquoted.example.com",
+        "quoted_external_module": '"http://quoted.example.org"',
+    }
+
+    settings = copy_fortran_file(data)
+    settings["extra_mods"] = [f"{key}: {value}" for key, value in extra_mods.items()]
+
+    project = create_project(settings)
+    mod_a = project.modules[0]
+    mod_b = project.modules[1]
+    mod_c = project.modules[2]
+    mod_d = project.modules[3]
+
+    assert mod_a.uses == set()
+    assert mod_b.uses == {mod_a}
+    assert mod_c.uses == {mod_a, mod_b}
+
+    mod_d_links = list(mod_d.uses)
+    for link in mod_d_links:
+        soup = BeautifulSoup(link, features="html.parser")
+        link = soup.a
+        expected_link = extra_mods[soup.text].strip(r"\"'")
+        assert link["href"] == expected_link, link
+
+
+def test_submodule_uses(copy_fortran_file):
+    """Check that module `USE`s are matched up correctly"""
+
+    data = """\
+    module mod_a
+    end module mod_a
+
+    submodule (mod_a) mod_b
+    end submodule mod_b
+
+    submodule (mod_a) mod_c
+    end submodule mod_c
+
+    submodule (mod_a:mod_c) mod_d
+    end submodule mod_d
+    """
+
+    extra_mods = {
+        "unquoted_external_module": "http://unquoted.example.com",
+        "quoted_external_module": '"http://quoted.example.org"',
+    }
+
+    settings = copy_fortran_file(data)
+    settings["extra_mods"] = [f"{key}: {value}" for key, value in extra_mods.items()]
+
+    project = create_project(settings)
+    mod_a = project.modules[0]
+    mod_b = project.submodules[0]
+    mod_c = project.submodules[1]
+    mod_d = project.submodules[2]
+
+    assert mod_a.uses == set()
+
+    assert mod_b.parent_submodule is None
+    assert mod_b.ancestor_module == mod_a
+
+    assert mod_c.parent_submodule is None
+    assert mod_c.ancestor_module == mod_a
+
+    assert mod_d.parent_submodule == mod_c
+    assert mod_d.ancestor_module == mod_a
