@@ -2,7 +2,7 @@ import shutil
 import sys
 import os
 import pathlib
-import re
+import copy
 from urllib.parse import urlparse
 import json
 
@@ -57,39 +57,51 @@ REMOTE_MODULES_JSON = [
 ]
 
 
-@pytest.fixture
-def copy_project_files(tmp_path):
-    this_dir = pathlib.Path(__file__).parent
-    shutil.copytree(
-        this_dir / "../../test_data/external_project", tmp_path / "external_project"
-    )
-    return tmp_path / "external_project"
-
-
 class MockResponse:
     @staticmethod
     def read():
         return json.dumps(REMOTE_MODULES_JSON).encode("utf-8")
 
 
-def test_external_project(copy_project_files, monkeypatch, restore_macros):
-    """Check that we can build external projects and get the links correct
+@pytest.fixture(scope="module")
+def monkeymodule(request):
+    """pytest won't let us use function-scope fixtures in module-scope
+    fixtures, so we need to reimplement this with module scope"""
+    mpatch = pytest.MonkeyPatch()
+    yield mpatch
+    mpatch.undo()
 
-    This is a rough-and-ready test that runs FORD via subprocess, and
-    so won't work unless FORD has been installed.
 
-    It also relies on access to the internet and an external URL out
-    of our control.
+@pytest.fixture(scope="module")
+def restore_macros_module():
+    """pytest won't let us use function-scope fixtures in module-scope
+    fixtures, so we need to reimplement this with module scope"""
+
+    old_macros = copy.copy(ford.utils._MACRO_DICT)
+    yield
+    ford.utils._MACRO_DICT = copy.copy(old_macros)
+
+
+@pytest.fixture(scope="module")
+def external_project(tmp_path_factory, monkeymodule, restore_macros_module):
+    """Generate the documentation for an "external" project and then
+    for a "top level" one that uses the first.
+
+    A remote external project is simulated through a mocked `urlopen`
+    which returns `REMOTE_MODULES_JSON`
 
     """
 
-    path = copy_project_files
+    this_dir = pathlib.Path(__file__).parent
+    path = tmp_path_factory.getbasetemp() / "external_project"
+    shutil.copytree(this_dir / "../../test_data/external_project", path)
+
     external_project = path / "external_project"
     top_level_project = path / "top_level_project"
 
     # Run FORD in the two projects
     # First project has "externalize: True" and will generate JSON dump
-    with monkeypatch.context() as m:
+    with monkeymodule.context() as m:
         os.chdir(external_project)
         m.setattr(sys, "argv", ["ford", "doc.md"])
         ford.run()
@@ -98,7 +110,7 @@ def test_external_project(copy_project_files, monkeypatch, restore_macros):
         return MockResponse()
 
     # Second project uses JSON from first to link to external modules
-    with monkeypatch.context() as m:
+    with monkeymodule.context() as m:
         os.chdir(top_level_project)
         m.setattr(sys, "argv", ["ford", "doc.md"])
         m.setattr(ford.utils, "urlopen", mock_open)
@@ -108,6 +120,14 @@ def test_external_project(copy_project_files, monkeypatch, restore_macros):
     # resolve correctly
     os.chdir("/")
 
+    return top_level_project, external_project
+
+
+def test_external_project(external_project):
+    """Check that we can build external projects and get the links correct"""
+
+    top_level_project, _ = external_project
+
     # Read generated HTML
     with open(top_level_project / "doc/program/top_level.html", "r") as f:
         top_program_html = BeautifulSoup(f.read(), features="html.parser")
@@ -116,7 +136,7 @@ def test_external_project(copy_project_files, monkeypatch, restore_macros):
     uses_box = top_program_html.find(string="Uses").parent.parent.parent
     links = {tag.text: tag.a["href"] for tag in uses_box("li", class_=None)}
 
-    assert len(links) == 2
+    assert len(links) == 3
     assert "external_module" in links
     local_url = urlparse(links["external_module"])
     assert pathlib.Path(local_url.path).is_file()
@@ -124,3 +144,22 @@ def test_external_project(copy_project_files, monkeypatch, restore_macros):
     assert "remote_module" in links
     remote_url = urlparse(links["remote_module"])
     assert remote_url.scheme == "https"
+
+
+def test_procedure_module_use_links_(external_project):
+    """Check that links to external modules used by functions are correct"""
+
+    top_level_project, _ = external_project
+
+    # Read generated HTML
+    with open(top_level_project / "doc/proc/abortcriteria_load.html", "r") as f:
+        procedure_html = BeautifulSoup(f.read(), features="html.parser")
+
+    # Find links to external modules
+    uses_box = procedure_html.find(string="Uses").parent.parent.parent
+    links = {tag.text: tag.a["href"] for tag in uses_box("li", class_=None)}
+
+    assert len(links) == 1
+    assert "external_module" in links
+    local_url = urlparse(links["external_module"])
+    assert pathlib.Path(local_url.path).is_file()
