@@ -637,6 +637,39 @@ def test_module_procedure_case(parse_fortran_file):
     assert module.interfaces[3].procedure.module
 
 
+def test_submodule_ancestors(parse_fortran_file):
+    """Check that submodule ancestors and parents are correctly identified"""
+
+    data = """\
+    module mod_a
+    end module mod_a
+
+    submodule (mod_a) mod_b
+    end submodule mod_b
+
+    submodule (mod_a) mod_c
+    end submodule mod_c
+
+    submodule (mod_a:mod_c) mod_d
+    end submodule mod_d
+    """
+
+    fortran_file = parse_fortran_file(data)
+
+    mod_b = fortran_file.submodules[0]
+    mod_c = fortran_file.submodules[1]
+    mod_d = fortran_file.submodules[2]
+
+    assert mod_b.parent_submodule is None
+    assert mod_b.ancestor_module == "mod_a"
+
+    assert mod_c.parent_submodule is None
+    assert mod_c.ancestor_module == "mod_a"
+
+    assert mod_d.parent_submodule == "mod_c"
+    assert mod_d.ancestor_module == "mod_a"
+
+
 @dataclass
 class ParsedType:
     vartype: str
@@ -811,32 +844,38 @@ class FakeVariable:
             "type(foo) :: bar = 42",
             [FakeVariable("bar", "type", proto=["foo", ""], initial="42")],
         ),
+        (
+            "class(foo) :: var1, var2",
+            [
+                FakeVariable("var1", "class", proto=["foo", ""]),
+                FakeVariable("var2", "class", proto=["foo", ""]),
+            ],
+        ),
+        (
+            "class(*) :: polymorphic",
+            [FakeVariable("polymorphic", "class", proto=["*", ""])],
+        ),
     ],
 )
 def test_line_to_variable(line, expected_variables):
     variables = line_to_variables(FakeSource(), line, "public", FakeParent())
+    attributes = expected_variables[0].__dict__
 
     for variable, expected in zip(variables, expected_variables):
-        for attr in [
-            "name",
-            "vartype",
-            "parent",
-            "attribs",
-            "intent",
-            "optional",
-            "permission",
-            "parameter",
-            "kind",
-            "strlen",
-            "proto",
-            "doc",
-            "points",
-            "initial",
-        ]:
+        for attr in attributes:
             variable_attr = getattr(variable, attr)
             expected_attr = getattr(expected, attr)
             assert variable_attr == expected_attr, attr
 
+    if len(expected_variables) > 1:
+        attributes.pop("parent")
+        for attr in attributes:
+            attribute = getattr(variable, attr)
+            if not isinstance(attribute, list):
+                continue
+            proto_ids = [id(getattr(variable, attr)) for variable in variables]
+            assert len(proto_ids) == len(set(proto_ids)), attr
+
 
 def test_markdown_header_bug286(parse_fortran_file):
     """Check that markdown headers work, issue #286"""
@@ -1016,3 +1055,119 @@ def test_markdown_source_settings(parse_fortran_file):
 
     assert subroutine.meta["source"]
     assert "with_source" in subroutine.src
+
+
+@pytest.mark.parametrize(
+    ["snippet", "expected_error", "expected_name"],
+    (
+        (
+            "program foo\n contains\n contains",
+            "Multiple CONTAINS",
+            "foo",
+        ),
+        (
+            "program foo\n interface bar\n contains",
+            "Unexpected CONTAINS",
+            "bar",
+        ),
+        ("end", "END statement", "test.f90"),
+        ("program foo\n module procedure bar", "Unexpected MODULE PROCEDURE", "foo"),
+        ("program foo\n module bar", "Unexpected MODULE", "foo"),
+        (
+            "program foo\n submodule (foo) bar \n end program foo",
+            "Unexpected SUBMODULE",
+            "program 'foo'",
+        ),
+        ("program foo\n program bar", "Unexpected PROGRAM", "foo"),
+        (
+            "program foo\n end program foo\n program bar \n end program bar",
+            "Multiple PROGRAM",
+            "test.f90",
+        ),
+        (
+            "program foo\n subroutine bar \n end subroutine \n end program",
+            "Unexpected SUBROUTINE",
+            "program 'foo'",
+        ),
+        (
+            "program foo\n integer function bar() \n end function bar\n end program foo",
+            "Unexpected FUNCTION",
+            "program 'foo'",
+        ),
+    ),
+)
+def test_bad_parses(snippet, expected_error, expected_name, parse_fortran_file):
+    with pytest.raises(ValueError) as e:
+        parse_fortran_file(snippet, dbg=False)
+
+    assert expected_error in e.value.args[0]
+    assert expected_name in e.value.args[0]
+
+
+def test_routine_iterator(parse_fortran_file):
+    data = """\
+    module foo
+      interface
+        module subroutine modsub1()
+        end subroutine modsub1
+        module subroutine modsub2()
+        end subroutine modsub2
+        module integer function modfunc1()
+        end function modfunc1
+        module integer function modfunc2()
+        end function modfunc2
+      end interface
+    contains
+      subroutine sub1()
+      end subroutine sub1
+      subroutine sub2()
+      end subroutine sub2
+      integer function func1()
+      end function func1
+      integer function func2()
+      end function func2
+    end module foo
+
+    submodule (foo) bar
+    contains
+      module subroutine modsub1
+      end subroutine modsub1
+      module subroutine modsub2
+      end subroutine modsub2
+      module procedure modfunc1
+      end procedure modfunc1
+      module procedure modfunc2
+      end procedure modfunc2
+
+      subroutine sub3()
+      end subroutine sub3
+      subroutine sub4()
+      end subroutine sub4
+      integer function func3()
+      end function func3
+      integer function func4()
+      end function func4
+    end submodule bar
+    """
+
+    fortran_file = parse_fortran_file(data)
+
+    module = fortran_file.modules[0]
+    assert sorted([proc.name for proc in module.routines]) == [
+        "func1",
+        "func2",
+        "sub1",
+        "sub2",
+    ]
+
+    submodule = fortran_file.submodules[0]
+    assert sorted([proc.name for proc in submodule.routines]) == [
+        "func3",
+        "func4",
+        "modfunc1",
+        "modfunc2",
+        "modsub1",
+        "modsub2",
+        "sub3",
+        "sub4",
+    ]
